function [xfilt,yfilt]=Filtfiltering(x,y,L,theta);

%This functions filters the data vectors x and y with a composite window of
%length L. Composite window of length is is a weighted average of theta of
%Ricker window of length ~L/2 and (1-theta) of Chebyshev window of length L
%L=Filter length. The filtered x vector is xfilt and the filtered y vector
%is yfilt
%
%Use: [xfilt,yfilt]=Filtering(x,y,L,theta);
%
%(c) Aymeric-Pierre B. Peyret 2010-2011

%w=barthannwin(L);
%w=hamming(L);

% xfilt=filter(w,[1],x);
% yfilt=filter(w,[1],y);

if nargin<4
    theta=0.6;
end

nanval=union(find(isnan(x)),find(isnan(y)));
nanboth=intersect(find(isnan(x)),find(isnan(y)));
notnanneither=setdiff(1:length(x),nanval);
nanx=setdiff(find(isnan(x)),nanboth);
nany=setdiff(find(isnan(y)),nanboth);

% msgbox('There are NaN values in your data. These values will be IGNORED.');

if ~isempty(nanval)
    originalx=x;
    originaly=y;
    x=x(notnanneither);
    y=y(notnanneither);
end

l=(L-1)/2;
ll=(l)/2;

%Creates ricker window and pads sides with zeros to match total window
%length
wr=[zeros(ll,1);ricker(l+1);zeros(ll,1)];wr=wr/sum(wr);

%creates Chebyshwv window
wc = window(@chebwin,L);wc=wc/sum(wc);

%Creates composite window
w=(1-theta)*wc+theta*wr;

%w = window(@boxcar,L);

%Pre-allocation of memory
xfilt=filtfilt(w,1,x);
yfilt=filtfilt(w,1,y);

if ~isempty(nanval)
    xtemp=xfilt;
    ytemp=yfilt;
    for i=1:length(originalx)
        if ismember(i,nanboth)
            xfilt(i)=NaN;
            yfilt(i)=NaN;
        elseif ismember(i,nanx)
            xfilt(i)=NaN;
            yfilt(i)=originaly(i);
        elseif ismember(i,nany)
            xfilt(i)=originalx(i);
            yfilt(i)=NaN;
        else
            xfilt(i)=xtemp(i-sum(find(nanval<i)));
            yfilt(i)=ytemp(i-sum(find(nanval<i)));
        end
    end
end

% xfilt=zeros(size(x));
% yfilt=zeros(size(y));
% xfilt(1:(L-1)/2)=NaN*zeros(size(xfilt(1:(L-1)/2)));
% yfilt(1:(L-1)/2)=NaN*zeros(size(yfilt(1:(L-1)/2)));
% xfilt(length(xfilt)-(L-1)/2+1:length(xfilt))=NaN*zeros(size(xfilt(length(xfilt)-(L-1)/2+1:length(xfilt))));
% yfilt(length(yfilt)-(L-1)/2+1:length(yfilt))=NaN*zeros(size(yfilt(length(yfilt)-(L-1)/2+1:length(yfilt))));
% 
% %Filters beginning edge of vectors with windows of increasing sizes
% for i=1:(L-1)/2
%     Li=2*i-1;
%     wci=window(@chebwin,Li);wci=wci/sum(wci);
%     wri=ricker(Li);wri=wri/sum(wri);
%     wi=(1-theta)*wci+theta*wri;
%     xfilt(i)=sum(x(1:Li).*wi/sum(wi));
%     yfilt(i)=sum(y(1:Li).*wi/sum(wi));
% end
% 
% %Filters ending edge of vectors with windows of decreasing sizes
% for i=length(x)-(L-1)/2+1:length(x)
%     j=length(x)-i+1;
%     Li=2*j-1;
%     wci=window(@chebwin,Li);wci=wci/sum(wci);
%     wri=ricker(Li);wri=wri/sum(wri);
%     wi=(1-theta)*wci+theta*wri;
%     xfilt(i)=sum(x(length(x)+1-Li:length(x)).*wi/sum(wi));
%     yfilt(i)=sum(y(length(y)+1-Li:length(y)).*wi/sum(wi));
% end
% 
% %Filters INSIDE of the vectors
% for i=1+(L-1)/2:length(x)-(L-1)/2
%     xfilt(i)=sum(x(i-(L-1)/2:i+(L-1)/2).*w/sum(w));
%     yfilt(i)=sum(y(i-(L-1)/2:i+(L-1)/2).*w/sum(w));
% end