%Pick Extrema automatically - currently does not work very well, if it works at all

function [d11,ex11,d12,ex12,d21,ex21,d22,ex22,v11,v12,v21,v22,slope,intercept,slopeortho,interceptortho,vardir,alpha1best,alpha2best,beta1best,beta2best]=PickEx(Distance1,var1,Distance2,var2,BoundaryMin,BoundaryMax,linlog1,linlog2,invornot,alpha1,alpha2,beta1,beta2,findbest,WLength1,WLength2,cororlag)

% INPUTS:
%
% Order: PickEx(Distance1,var1,Distance2,var2,BoundaryMin,BoundaryMax,linlog1,linlog2,invornot,alpha1,alpha2,beta1,beta2,findbest,WLength1,WLength2,cororlag)
%
% Distance1: Distance corresponding to points of var1, monotonously increasing
% var1: variable 1
% Distance2: Distance corresponding to points of var2, monotonously increasing
% var2: variable 2
% BoundaryMin: Minimum distance wherecode is applied. Use <0 to bypass
% BoundaryMax: Maximum distance wherecode is applied. Use <0 to bypass
% linlog1: variable 1 scale: 0 for linear, 1 for logarithmic
% linlog2: variable 2 scale: 0 for linear, 1 for logarithmic
% Note that the radius of curvature is almost always log-scaled
% invornot: 1 for Positive correlation, -1 for negative correlation, 0 for automatic
% alpha1: Coefficient for low-pass filter to help determine minima of var1
% alpha2: Coefficient for low-pass filter to help determine maxima of var1
% beta1: Coefficient for low-pass filter to help determine minima of var2
% beta2: Coefficient for low-pass filter to help determine maxima of var2
% findbest: 0 runs code with given values of alpha(1/2), beta(1/2); 1 finds
%   values of alpha(1/2), beta(1/2) that maximize the number of extrema
%   found. Range tested is given values of alpha(1/2) and beta(1/2) +/- 0.25
%   by increments of 0.01
% WLength1: Length of low-pass filter for variable 1. Default 5% of var1
% WLength2: Length of low-pass filter for variable 2. Default 5% of var2
% cororlag: When using automatic mode, 1 if correlation coefficient decides
%   of correlation type and 0 if minimum lag between variables decides of
%   correlation type
% Low-pass filter is *always* Chebyshev window here.
% Inputs var1 and var2 WITHOUT ANY NaN value (Only 1st and last points allowed)!!!
% Only COLUMN vectors please!
%
% OUTPUTS:
%
% Order: [d11,ex11,d12,ex12,d21,ex21,d22,ex22,v11,v12,v21,v22,slope,intercept,slopeortho,interceptortho,vardir,alpha1best,alpha2best,beta1best,beta2best]
%
% ex11: For variable 1, indices of minima matched with (minima/maxima) of variable 2
% ex12: For variable 2, indices of (minima/maxima) matched with minima of variable 1
% ex21: For variable 1, indices of maxima matched with (maxima/minima) of variable 2
% ex22: For variable 2, indices of (maxima/minima) matched with maxima of variable 1
% vardir
% alpha1best: if findbest=0, alpha1; Otherwise, value of alpha1 maximizing the number of found extrema
% alpha2best: if findbest=0, alpha2; Otherwise, value of alpha2 maximizing the number of found extrema
% beta1best: if findbest=0, beta1; Otherwise, value of beta1 maximizing the number of found extrema
% beta2best: if findbest=0, beta2; Otherwise, value of beta2 maximizing the number of found extrema


var1all=var1;Distance1all=Distance1;
var2all=var2;Distance2all=Distance2;

toaddindex1=0;
toaddindex2=0;

if BoundaryMin>0
    min1=min(find(Distance1>=BoundaryMin));
    min2=min(find(Distance2>=BoundaryMin));
    var1=var1(min1:end);
    var2=var2(min2:end);
    Distance1=Distance1(min1:end);
    Distance2=Distance2(min2:end);
    toaddindex1=min1-1;
    toaddindex2=min2-1;
end
if (BoundaryMax>0)%&&(BoundaryMin>0)
    max1=max(find(Distance1<=BoundaryMax));
    max2=max(find(Distance2<=BoundaryMax));
    var1=var1(1:max1);
    var2=var2(1:max2);
    Distance1=Distance1(1:max1);
    Distance2=Distance2(1:max2);
end

var1ori=var1;l1=length(var1);
var2ori=var2;l2=length(var2);

%l1 should be =length(Distance1) and l2 = length(Distance2)

epsilon=10^(-15);%substituted to 0 when taking log of data, to avoid NaN values that mess up with the low-pass filter

isnan1=find(isnan(var1));isnan2=find(isnan(var2));

if ~isempty(setdiff(isnan1,[1 l1])), display('Variable 1 contains NaN value. Please correct before running this function'); end
if ~isempty(setdiff(isnan2,[1 l2])), display('Variable 2 contains NaN value. Please correct before running this function'); end
if ismember(l1,isnan1), var1=var1(1:end-1); end
if ismember(1,isnan1), var1=var1(2:end);end
if ismember(l2,isnan2), var2=var2(1:end-1); end
if ismember(1,isnan2), var2=var2(2:end);end

var1mod=var1;var2mod=var2;

if linlog1, var1(var1==0)=min(epsilon,min(var1(var1~=0))/10); var1=log10(abs(var1)); end
if linlog2, var2(var2==0)=min(epsilon,min(var2(var2~=0))/10); var2=log10(abs(var2)); end

if (nargin<15 || WLength1<=0), WLength1=round(length(var1)/20); end
if (nargin<16 || WLength2<=0), WLength2=round(length(var2)/20); end
if nargin<17 cororlag=1;end


window1=chebwin(WLength1)/(sum(chebwin(WLength1)));
window2=chebwin(WLength2)/(sum(chebwin(WLength2)));

%Because filtering assumes periodic, mirror image is added to vectors then
%taken off after filtering; this helps a bit..
Var1low=filtfilt(window1,1,[var1;flipud(var1)]);Var1low=Var1low(1:end/2);
Var2low=filtfilt(window2,1,[var2;flipud(var2)]);Var2low=Var2low(1:end/2);

%Now, after filtering, we can put back NaN values and log(0) values as NaN

if linlog1, var1(var1mod==0)=NaN; end
if linlog2, var2(var2mod==0)=NaN; end
if ismember(l1,isnan1), var1=[var1;NaN]; end
if ismember(1,isnan1), var1=[1;var1]; end
if ismember(l2,isnan2), var2=[var2;NaN]; end
if ismember(1,isnan2), var2=[1;var2]; end

if linlog1, Var1low(var1mod==0)=NaN; end
if linlog2, Var2low(var2mod==0)=NaN; end
if ismember(l1,isnan1), Var1low=[Var1low;NaN]; end
if ismember(1,isnan1), Var1low=[1;Var1low]; end
if ismember(l2,isnan2), Var2low=[Var2low;NaN]; end
if ismember(1,isnan2), Var2low=[1;Var2low]; end

Var1low=Var1low*nanmean(var1)/nanmean(Var1low);
Var2low=Var2low*nanmean(var2)/nanmean(Var2low);

%Go back to functions without log if necessary

if linlog1, var1=10.^(var1);Var1low=10.^(Var1low); end
if linlog2, var2=10.^(var2);Var2low=10.^(Var2low); end

[minvar1,minvar2,maxvar1,maxvar2,alpha1best,alpha2best,beta1best,beta2best]=findextrema(var1,Var1low,var2,Var2low,alpha1,alpha2,beta1,beta2,findbest);

% minvarX(...,1): Local minima = varX(minvarX(...,2))
% minvarX(...,2): Corresponding indices to local minima
% maxvarX(...,1): Local maxima = varX(maxvarX(...,2))
% maxvarX(...,2): Corresponding indices to local maxima


% [min(Distance1)/1000 max(Distance1)/1000]
% [min(Distance2)/1000 max(Distance2)/1000]

%%%%%%%%%Correlations are below; need to switch 1 and 2 if 1 has more picked-up extrema than 2
lm1=length(minvar1);
lm2=length(minvar2);
lM1=length(maxvar1);
lM2=length(maxvar2);

Distmin1=Distance1(minvar1(:,2));Distmax1=Distance1(maxvar1(:,2));
Distmin2=Distance2(minvar2(:,2));Distmax2=Distance2(maxvar2(:,2));

if invornot==1
    DistmatA=Distmin1*ones(1,length(Distmin2))-ones(length(Distmin1),1)*Distmin2';
    DistmatB=Distmax1*ones(1,length(Distmax2))-ones(length(Distmax1),1)*Distmax2';
    if lm1<=lm2
        mindistA=zeros(length(minvar1),5);
        for i=1:length(minvar1)
            mindistA(i,1)=minvar1(i,2);
            mindistA(i,2)=Distance1(mindistA(i,1));
            mindistA(i,3)=minvar2(find(abs(DistmatA(i,:))==min(abs(DistmatA(i,:)))),2);
            mindistA(i,4)=Distance2(mindistA(i,3));
            mindistA(i,5)=DistmatA(i,find(abs(DistmatA(i,:))==min(abs(DistmatA(i,:)))));
        end
        mindistA=DelDuplicates(mindistA,3);
    else
        mindistA=zeros(length(minvar2),5);
        for i=1:length(minvar2)
            mindistA(i,1)=minvar1(find(abs(DistmatA(:,i))==min(abs(DistmatA(:,i)))),2);
            mindistA(i,2)=Distance1(mindistA(i,1));
            mindistA(i,3)=minvar2(i,2);
            mindistA(i,4)=Distance2(mindistA(i,3));
            mindistA(i,5)=DistmatA(find(abs(DistmatA(:,i))==min(abs(DistmatA(:,i)))),i);
        end
        mindistA=DelDuplicates(mindistA,1);
    end
    if lM1<=lM2
        mindistB=zeros(length(maxvar1),5);
        for i=1:length(maxvar1)
            mindistB(i,1)=maxvar1(i,2);
            mindistB(i,2)=Distance1(mindistB(i,1));
            mindistB(i,3)=maxvar2(find(abs(DistmatB(i,:))==min(abs(DistmatB(i,:)))),2);
            mindistB(i,4)=Distance2(mindistB(i,3));
            mindistB(i,5)=DistmatB(i,find(abs(DistmatB(i,:))==min(abs(DistmatB(i,:)))));
        end
        mindistB=DelDuplicates(mindistB,3);
    else
        mindistB=zeros(length(maxvar2),5);
        for i=1:length(maxvar2)
            mindistB(i,1)=maxvar1(find(abs(DistmatB(:,i))==min(abs(DistmatB(:,i)))),2);
            mindistB(i,2)=Distance1(mindistB(i,1));
            mindistB(i,3)=maxvar2(i,2);
            mindistB(i,4)=Distance2(mindistB(i,3));
            mindistB(i,5)=DistmatB(find(abs(DistmatB(:,i))==min(abs(DistmatB(:,i)))),i);
        end
        mindistB=DelDuplicates(mindistB,1);
    end
elseif invornot==-1
    DistmatA=Distmin1*ones(1,length(Distmax2))-ones(length(Distmin1),1)*Distmax2';
    DistmatB=Distmax1*ones(1,length(Distmin2))-ones(length(Distmax1),1)*Distmin2';
    if lm1<=lM2
        mindistA=zeros(length(minvar1),5);
        for i=1:length(minvar1)
            mindistA(i,1)=minvar1(i,2);
            mindistA(i,2)=Distance1(mindistA(i,1));
            mindistA(i,3)=maxvar2(find(abs(DistmatA(i,:))==min(abs(DistmatA(i,:)))),2);
            mindistA(i,4)=Distance2(mindistA(i,3));
            mindistA(i,5)=DistmatA(i,find(abs(DistmatA(i,:))==min(abs(DistmatA(i,:)))));
        end
        mindistA=DelDuplicates(mindistA,3);
    else
        mindistA=zeros(length(maxvar2),5);
        for i=1:length(maxvar2)
            mindistA(i,1)=minvar1(find(abs(DistmatA(:,i))==min(abs(DistmatA(:,i)))),2);
            mindistA(i,2)=Distance1(mindistA(i,1));
            mindistA(i,3)=maxvar2(i,2);
            mindistA(i,4)=Distance2(mindistA(i,3));
            mindistA(i,5)=DistmatA(find(abs(DistmatA(:,i))==min(abs(DistmatA(:,i)))),i);
        end
        mindistA=DelDuplicates(mindistA,1);
    end
    if lM1<=lm2
        mindistB=zeros(length(maxvar1),5);
        for i=1:length(maxvar1)
            mindistB(i,1)=maxvar1(i,2);
            mindistB(i,2)=Distance1(mindistB(i,1));
            mindistB(i,3)=minvar2(find(abs(DistmatB(i,:))==min(abs(DistmatB(i,:)))),2);
            mindistB(i,4)=Distance2(mindistB(i,3));
            mindistB(i,5)=DistmatB(i,find(abs(DistmatB(i,:))==min(abs(DistmatB(i,:)))));
        end
        mindistB=DelDuplicates(mindistB,3);
    else
        mindistB=zeros(length(minvar2),5);
        for i=1:length(minvar2)
            mindistB(i,1)=maxvar1(find(abs(DistmatB(:,i))==min(abs(DistmatB(:,i)))),2);
            mindistB(i,2)=Distance1(mindistB(i,1));
            mindistB(i,3)=minvar2(i,2);
            mindistB(i,4)=Distance2(mindistB(i,3));
            mindistB(i,5)=DistmatB(find(abs(DistmatB(:,i))==min(abs(DistmatB(:,i)))),i);
        end
        mindistB=DelDuplicates(mindistB,1);
    end
else
    DistmatA=Distmin1*ones(1,length(Distmin2))-ones(length(Distmin1),1)*Distmin2';
    DistmatB=Distmax1*ones(1,length(Distmax2))-ones(length(Distmax1),1)*Distmax2';
    DistmatC=Distmin1*ones(1,length(Distmax2))-ones(length(Distmin1),1)*Distmax2';
    DistmatD=Distmax1*ones(1,length(Distmin2))-ones(length(Distmax1),1)*Distmin2';
    %     mindistA=zeros(length(minvar1),5);mindistB=zeros(length(maxvar1),5);
    %     mindistC=zeros(length(minvar1),5);mindistD=zeros(length(maxvar1),5);
    if lm1<=lm2
        mindistA=zeros(length(minvar1),5);
        for i=1:length(minvar1)
            mindistA(i,1)=minvar1(i,2);
            mindistA(i,2)=Distance1(mindistA(i,1));
            mindistA(i,3)=minvar2(find(abs(DistmatA(i,:))==min(abs(DistmatA(i,:)))),2);
            mindistA(i,4)=Distance2(mindistA(i,3));
            mindistA(i,5)=DistmatA(i,find(abs(DistmatA(i,:))==min(abs(DistmatA(i,:)))));
        end
        mindistA=DelDuplicates(mindistA,3);
    else
        mindistA=zeros(length(minvar2),5);
        for i=1:length(minvar2)
            mindistA(i,1)=minvar1(find(abs(DistmatA(:,i))==min(abs(DistmatA(:,i)))),2);
            mindistA(i,2)=Distance1(mindistA(i,1));
            mindistA(i,3)=minvar2(i,2);
            mindistA(i,4)=Distance2(mindistA(i,3));
            mindistA(i,5)=DistmatA(find(abs(DistmatA(:,i))==min(abs(DistmatA(:,i)))),i);
        end
        mindistA=DelDuplicates(mindistA,1);
    end
    if lM1<=lM2
        mindistB=zeros(length(maxvar1),5);
        for i=1:length(maxvar1)
            mindistB(i,1)=maxvar1(i,2);
            mindistB(i,2)=Distance1(mindistB(i,1));
            mindistB(i,3)=maxvar2(find(abs(DistmatB(i,:))==min(abs(DistmatB(i,:)))),2);
            mindistB(i,4)=Distance2(mindistB(i,3));
            mindistB(i,5)=DistmatB(i,find(abs(DistmatB(i,:))==min(abs(DistmatB(i,:)))));
        end
        mindistB=DelDuplicates(mindistB,3);
    else
        mindistB=zeros(length(maxvar2),5);
        for i=1:length(maxvar2)
            mindistB(i,1)=maxvar1(find(abs(DistmatB(:,i))==min(abs(DistmatB(:,i)))),2);
            mindistB(i,2)=Distance1(mindistB(i,1));
            mindistB(i,3)=maxvar2(i,2);
            mindistB(i,4)=Distance2(mindistB(i,3));
            mindistB(i,5)=DistmatB(find(abs(DistmatB(:,i))==min(abs(DistmatB(:,i)))),i);
        end
        mindistB=DelDuplicates(mindistB,1);
    end
    if lm1<=lM2
        mindistC=zeros(length(minvar1),5);
        for i=1:length(minvar1)
            mindistC(i,1)=minvar1(i,2);
            mindistC(i,2)=Distance1(mindistC(i,1));
            mindistC(i,3)=maxvar2(find(abs(DistmatC(i,:))==min(abs(DistmatC(i,:)))),2);
            mindistC(i,4)=Distance2(mindistC(i,3));
            mindistC(i,5)=DistmatC(i,find(abs(DistmatC(i,:))==min(abs(DistmatC(i,:)))));
        end
        mindistC=DelDuplicates(mindistC,3);
    else
        mindistC=zeros(length(maxvar2),5);
        for i=1:length(maxvar2)
            mindistC(i,1)=minvar1(find(abs(DistmatC(:,i))==min(abs(DistmatC(:,i)))),2);
            mindistC(i,2)=Distance1(mindistC(i,1));
            mindistC(i,3)=maxvar2(i,2);
            mindistC(i,4)=Distance2(mindistC(i,3));
            mindistC(i,5)=DistmatC(find(abs(DistmatC(:,i))==min(abs(DistmatC(:,i)))),i);
        end
        mindistC=DelDuplicates(mindistC,1);
    end
    if lM1<=lm2
        mindistD=zeros(length(maxvar1),5);
        for i=1:length(maxvar1)
            mindistD(i,1)=maxvar1(i,2);
            mindistD(i,2)=Distance1(mindistD(i,1));
            mindistD(i,3)=minvar2(find(abs(DistmatD(i,:))==min(abs(DistmatD(i,:)))),2);
            mindistD(i,4)=Distance2(mindistD(i,3));
            mindistD(i,5)=DistmatD(i,find(abs(DistmatD(i,:))==min(abs(DistmatD(i,:)))));
        end
        mindistD=DelDuplicates(mindistD,3);
    else
        mindistD=zeros(length(minvar2),5);
        for i=1:length(minvar2)
            mindistD(i,1)=maxvar1(find(abs(DistmatD(:,i))==min(abs(DistmatD(:,i)))),2);
            mindistD(i,2)=Distance1(mindistD(i,1));
            mindistD(i,3)=minvar2(i,2);
            mindistD(i,4)=Distance2(mindistD(i,3));
            mindistD(i,5)=DistmatD(find(abs(DistmatD(:,i))==min(abs(DistmatD(:,i)))),i);
        end
        mindistD=DelDuplicates(mindistD,1);
    end
end

% minvarX(...,1): Local minima = varX(minvarX(...,2))
% minvarX(...,2): Corresponding indices to local minima
% maxvarX(...,1): Local maxima = varX(maxvarX(...,2))
% maxvarX(...,2): Corresponding indices to local maxima
% mindistX(...,1): Indices of local minima/maxima
% mindistX(...,2): Distances to local minima/maxima
% mindistX(...,3): Indices of corresponding local minima/maxima
% mindistX(...,4): Distances to corresponding local minima/maxima
% mindistX(...,5): Lag between corresponding points
%A: minvar1 minvar2
%B: maxvar1 maxvar2
%C: minvar1 maxvar2
%D: maxvar1 minvar2

if abs(invornot)==1
    ex11=mindistA(:,1);v11=var1(ex11);ex11=ex11+toaddindex1;d11=mindistA(:,2);
    ex12=mindistA(:,3);v12=var2(ex12);ex12=ex12+toaddindex2;d12=mindistA(:,4);
    ex21=mindistB(:,1);v21=var1(ex21);ex21=ex21+toaddindex1;d21=mindistB(:,2);
    ex22=mindistB(:,3);v22=var2(ex22);ex22=ex22+toaddindex2;d22=mindistB(:,4);
    vardir=invornot;
%     (Distance1all(ex11)-d11)'
%     (Distance2all(ex12)-d12)'
%     (Distance1all(ex21)-d21)'
%     (Distance2all(ex22)-d22)'
else
    mindistA';
    mindistB';
    mindistC';
    mindistD';
    %Need to choose which correlation is best...
    var1AB=var1([mindistA(:,1);mindistB(:,1)]);%
    var2AB=var2([mindistA(:,3);mindistB(:,3)]);%
    var1CD=var1([mindistC(:,1);mindistD(:,1)]);%
    var2CD=var2([mindistC(:,3);mindistD(:,3)]);%
    LagAB=abs([mindistA(:,5);mindistB(:,5)]);
    LagCD=abs([mindistC(:,5);mindistD(:,5)]);
    AvLagAB=nanmean(LagAB);AvLagCD=nanmean(LagCD);%Norm 1
    AvLagAB=sqrt(nansum(LagAB.^2));AvLagCD=sqrt(nansum(LagCD.^2));%norm 2
    AvLagAB=nanmax(LagAB);AvLagCD=nanmax(LagCD);%norm inf
    %Computes correlation coefficient between 2x2 sets of variables
    if linlog1
        if linlog2
            %             ccmatAB=corrcoef(log10(var1AB),log10(var2AB));ccAB=(ccmatAB(1,2))^2;
            %             ccmatCD=corrcoef(log10(var1CD),log10(var2CD));ccCD=(ccmatCD(1,2))^2;
            ccAB=corr2(log10(var1AB),log10(var2AB))^2;
            ccCD=corr2(log10(var1CD),log10(var2CD))^2;
        else
            %             ccmatAB=corrcoef(log10(var1AB),var2AB);ccAB=(ccmatAB(1,2))^2;
            %             ccmatCD=corrcoef(log10(var1CD),var2CD);ccCD=(ccmatCD(1,2))^2;
            ccAB=corr2(log10(var1AB),var2AB)^2;
            ccCD=corr2(log10(var1CD),var2CD)^2;
        end
    else
        if linlog2
            %             ccmatAB=corrcoef(var1AB,log10(var2AB));ccAB=(ccmatAB(1,2))^2;
            %             ccmatCD=corrcoef(var1CD,log10(var2CD));ccCD=(ccmatCD(1,2))^2;
            ccAB=corr2(var1AB,log10(var2AB))^2;
            ccCD=corr2(var1CD,log10(var2CD))^2;
        else
            %             ccmatAB=corrcoef(var1AB,var2AB);ccAB=(ccmatAB(1,2))^2;
            %             ccmatCD=corrcoef(var1CD,var2CD);ccCD=(ccmatCD(1,2))^2;
            ccAB=corr2(var1AB,var2AB)^2;
            ccCD=corr2(var1CD,var2CD)^2;
        end
    end
    if ~cororlag
        comp1=AvLagAB;comp2=AvLagCD;
        display('Choice of correlation type using minimum lag');
    else
        comp1=ccCD;comp2=ccAB;
        display('Choice of correlation type using maximum correlation coefficient');
    end
%     abs([100*(1-AvLagCD/AvLagAB) 100*(1-ccAB/ccCD)])
    if ((ccCD<ccAB) & (AvLagAB>AvLagCD)) || ((ccCD>ccAB) & (AvLagAB<AvLagCD))
        display('Correlation coefficient and minimum total lag indicate opposite trends!');
    end
    if comp1<comp2%Positive correlation is better, less lag or better correlation coef
        display('Positive correlation is better than negative correlation');
        ex11=mindistA(:,1);v11=var1(ex11);ex11=ex11+toaddindex1;d11=mindistA(:,2);
        ex12=mindistA(:,3);v12=var2(ex12);ex12=ex12+toaddindex2;d12=mindistA(:,4);
        ex21=mindistB(:,1);v21=var1(ex21);ex21=ex21+toaddindex1;d21=mindistB(:,2);
        ex22=mindistB(:,3);v22=var2(ex22);ex22=ex22+toaddindex2;d22=mindistB(:,4);
        vardir=1;
    else%Negative correlation is better, less lag
        display('Negative correlation is better than positive correlation');
        ex11=mindistC(:,1);v11=var1(ex11);ex11=ex11+toaddindex1;d11=mindistC(:,2);
        ex12=mindistC(:,3);v12=var2(ex12);ex12=ex12+toaddindex2;d12=mindistC(:,4);
        ex21=mindistD(:,1);v21=var1(ex21);ex21=ex21+toaddindex1;d21=mindistD(:,2);
        ex22=mindistD(:,3);v22=var2(ex22);ex22=ex22+toaddindex2;d22=mindistD(:,4);
        vardir=-1;
    end
%     (Distance1all(ex11)-d11)'
%     (Distance2all(ex12)-d12)'
%     (Distance1all(ex21)-d21)'
%     (Distance2all(ex22)-d22)'
end

% find((Distance1all(ex11)-d11)'~=0)
% find((Distance2all(ex12)-d12)'~=0)
% find((Distance1all(ex21)-d21)'~=0)
% find((Distance2all(ex22)-d22)'~=0)


var1all=[v11;v21];var2all=[v12;v22];
var1alllog=log10(var1all);var2alllog=log10(var2all);
okpoint=intersect(find(~isnan(var1alllog)),find(~isnan(var2alllog)));
union(find(isnan(var1alllog)),find(isnan(var2alllog)));
var1alllog=var1alllog(okpoint);
var2alllog=var2alllog(okpoint);

if linlog1&&linlog2
    [P,S]=polyfit(var1alllog,var2alllog,1);
    [Portho,Sortho,Resortho]=OrthoFit(var1alllog,var2alllog);
    slope=P(1);
    intercept=10^(P(2));
    slopeortho=Portho;
    interceptortho=10^Sortho;
    display(['Least-square fit var2=f(var1): var2= ' num2str(rounddec(intercept,2)) ' x var1 ^{' num2str(rounddec(slope,2)) '}.']);
    display(['Orthogonal least-square fit var2=f(var1): var2= ' num2str(rounddec(interceptortho,2)) ' x var1 ^{' num2str(rounddec(slopeortho,2)) '}.']);
elseif linlog1&&~linlog2%var2=alpha*log10(var1)+beta;
    [P,S]=polyfit(var1alllog,var2all,1);
    [Portho,Sortho,Resortho]=OrthoFit(var1alllog,var2all);
    slope=P(1);
    intercept=P(2);
    slopeortho=Portho;
    interceptortho=Sortho;
    display(['Least-square fit var2=f(var1): var2= ' num2str(rounddec(slope,2)) ' x log_{10}(var1) + ' num2str(rounddec(intercept,2)) '.']);
    display(['Orthogonal least-square fit var2=f(var1): var2= ' num2str(rounddec(slopeortho,2)) ' x log_{10}(var1} +' num2str(rounddec(interceptortho,2)) '.']);
elseif linlog2&&~linlog1%var2=10^(alpha*var1)* 10^beta;
    [P,S]=polyfit(var1all,var2alllog,1);
    [Portho,Sortho,Resortho]=OrthoFit(var1all,var2alllog);
    slope=P(1);
    intercept=10^(P(2));
    slopeortho=Portho;
    interceptortho=10^Sortho;
    display(['Least-square fit var2=f(var1): var2= ' num2str(rounddec(intercept,2)) ' x var1 ^{' num2str(rounddec(slope,2)) '}.']);
    display(['Orthogonal least-square fit var2=f(var1): var2= ' num2str(rounddec(interceptortho,2)) ' x var1 ^{' num2str(rounddec(slopeortho,2)) '}.']);
else
    [P,S]=polyfit(var1all,var2all,1);
    [Portho,Sortho,Resortho]=OrthoFit(var1all,var2all);
    slope=P(1);
    intercept=P(2);
    slopeortho=Portho;
    interceptortho=Sortho;
    display(['Least-square fit var2=f(var1): var2= ' num2str(rounddec(slope,2)) ' x var1 + ' num2str(rounddec(intercept,2)) '.']);
    display(['Orthogonal least-square fit var2=f(var1): var2= ' num2str(rounddec(slopeortho,2)) ' x var1 +' num2str(rounddec(interceptortho,2)) '.']);
end

% uncomment following lines to plot curves with extrema (nothing more)
% absvar1=abs(var1);absvar2=abs(var2);
% figure;
% plot(Distance1,var1);hold on;plot(Distance1,Var1low,'r--');
% plot(Distance1(maxvar1(:,2)),absvar1(maxvar1(:,2)),'go');
% if linlog1, set(gca,'YScale','log'); end
%
% figure;
% plot(Distance2,var2);hold on;plot(Distance2,Var2low,'r--');
% plot(Distance2(maxvar2(:,2)),absvar2(maxvar2(:,2)),'go');
% if linlog2, set(gca,'YScale','log'); end
%
% figure;
% plot(Distance1,var1);hold on;plot(Distance1,Var1low,'r--');
% plot(Distance1(minvar1(:,2)),absvar1(minvar1(:,2)),'go');
% if linlog1, set(gca,'YScale','log'); end
%
% figure;
% plot(Distance2,var2);hold on;plot(Distance2,Var2low,'r--');
% plot(Distance2(minvar2(:,2)),absvar2(minvar2(:,2)),'go');
% if linlog2, set(gca,'YScale','log'); end

%%%%%%%%%%
function [minvar1,minvar2,maxvar1,maxvar2,alpha1best,alpha2best,beta1best,beta2best]=findextrema(var1,Var1low,var2,Var2low,alpha1,alpha2,beta1,beta2,swipecoef)

absvar1=abs(var1);absvar2=abs(var2);
l1=length(var1);
l2=length(var2);

if ~swipecoef
    alpha1best=alpha1;alpha2best=alpha2;
    beta1best=beta1;beta2best=beta2;
    
    var1less=find((abs(var1)-Var1low/alpha1)<0);
    var1skips=find(diff(var1less)>1)';
    var1lessint=var1less(var1skips);
    var1lessint2=var1less(1+var1skips);
    var1lessint=[var1lessint;l1];
    var1lessint2=[min(var1less);var1lessint2];
    minvar1=zeros(length(var1lessint),2);
    for i=1:length(var1lessint)
        partvar1=absvar1(var1lessint2(i):var1lessint(i));
        minvar1(i,1)=nanmin(partvar1);minvar1(i,2)=find(partvar1==minvar1(i,1))-1+var1lessint2(i);
    end
    
    var1more=find((abs(var1)-Var1low*alpha2)>0);
    var1moreskips=find(diff(var1more)>1)';
    var1moreint=var1more(var1moreskips);
    var1moreint2=var1more(1+var1moreskips);
    var1moreint=[var1moreint;l1];
    var1moreint2=[min(var1more);var1moreint2];
    maxvar1=zeros(length(var1moreint),2);
    for i=1:length(var1moreint)
        partvar1=absvar1(var1moreint2(i):var1moreint(i));
        maxvar1(i,1)=nanmax(partvar1);maxvar1(i,2)=find(partvar1==maxvar1(i,1))-1+var1moreint2(i);
    end
    
    var2less=find((abs(var2)-Var2low/beta1)<0);
    var2skips=find(diff(var2less)>1)';
    var2lessint=var2less(var2skips);
    var2lessint2=var2less(1+var2skips);
    var2lessint=[var2lessint;l2];
    var2lessint2=[min(var2less);var2lessint2];
    minvar2=zeros(length(var2lessint),2);
    for i=1:length(var2lessint)
        partvar2=absvar2(var2lessint2(i):var2lessint(i));
        minvar2(i,1)=nanmin(partvar2);
        minvar2(i,2)=find(partvar2==minvar2(i,1))-1+var2lessint2(i);
    end
    
    var2more=find((abs(var2)-Var2low*beta2)>0);
    var2moreskips=find(diff(var2more)>1)';
    var2moreint=var2more(var2moreskips);
    var2moreint2=var2more(1+var2moreskips);
    var2moreint=[var2moreint;l2];
    var2moreint2=[min(var2more);var2moreint2];
    maxvar2=zeros(length(var2moreint),2);
    for i=1:length(var2moreint)
        partvar2=absvar2(var2moreint2(i):var2moreint(i));
        maxvar2(i,1)=nanmax(partvar2);
        maxvar2(i,2)=find(partvar2==maxvar2(i,1))-1+var2moreint2(i);
    end
else
    swipeval=-0.25:0.01:0.25;lswipe=length(swipeval);midpoint=find(swipeval==0);sizevar=zeros(lswipe,4);
    alpha1swipe=alpha1+swipeval;
    alpha2swipe=alpha2+swipeval;
    beta1swipe=beta1+swipeval;
    beta2swipe=beta2+swipeval;
    
    var1lowalpha1=absvar1*ones(size(swipeval))-(Var1low*ones(size(swipeval))).*(ones(size(var1))*(1./alpha1swipe));
    var1lowalpha2=absvar1*ones(size(swipeval))-(Var1low*ones(size(swipeval))).*(ones(size(var1))*alpha2swipe);
    var2lowbeta1=absvar2*ones(size(swipeval))-(Var2low*ones(size(swipeval))).*(ones(size(var2))*(1./beta1swipe));
    var2lowbeta2=absvar2*ones(size(swipeval))-(Var2low*ones(size(swipeval))).*(ones(size(var2))*beta2swipe);
    
    for i=1:lswipe
        var1less=find(var1lowalpha1(:,i)<0);sizevar(i,1)=1+length(find(diff(var1less)>1)');
        var1more=find(var1lowalpha2(:,i)>0);sizevar(i,2)=1+length(find(diff(var1more)>1)');
        var2less=find(var2lowbeta1(:,i)<0);sizevar(i,3)=1+length(find(diff(var2less)>1)');
        var2more=find(var2lowbeta2(:,i)>0);sizevar(i,4)=1+length(find(diff(var2more)>1)');
    end
    
    maxfound1less=find(sizevar(:,1)==max(sizevar(:,1)));
    maxfound1more=find(sizevar(:,2)==max(sizevar(:,2)));
    maxfound2less=find(sizevar(:,3)==max(sizevar(:,3)));
    maxfound2more=find(sizevar(:,4)==max(sizevar(:,4)));
    
    mf1l=find(abs(maxfound1less-midpoint)==min(abs(maxfound1less-midpoint)));
    mf1m=find(abs(maxfound1more-midpoint)==min(abs(maxfound1more-midpoint)));
    mf2l=find(abs(maxfound2less-midpoint)==min(abs(maxfound2less-midpoint)));
    mf2m=find(abs(maxfound2more-midpoint)==min(abs(maxfound2more-midpoint)));
    
    if length(mf1l)~=1
        whichmf1l=find(abs(1./alpha1swipe(maxfound1less(mf1l))-1)==min(abs(1./alpha1swipe(maxfound1less(mf1l))-1)));
        mf1l=mf1l(whichmf1l);
    end
    
    if length(mf1m)~=1
        whichmf1m=find(abs(alpha2swipe(maxfound1more(mf1m))-1)==min(abs(alpha2swipe(maxfound1more(mf1m))-1)));
        mf1m=mf1m(whichmf1m);
    end
    
    if length(mf2l)~=1
        whichmf2l=find(abs(1./beta1swipe(maxfound2less(mf2l))-1)==min(abs(1./beta1swipe(maxfound2less(mf2l))-1)));
        mf2l=mf2l(whichmf2l);
    end
    
    if length(mf2m)~=1
        whichmf2m=find(abs(beta2swipe(maxfound2more(mf2m))-1)==min(abs(beta2swipe(maxfound2more(mf2m))-1)));
        mf2m=mf2m(whichmf2m);
    end
    
    alpha1best=alpha1swipe(maxfound1less(mf1l));
    alpha2best=alpha2swipe(maxfound1more(mf1m));
    beta1best=beta1swipe(maxfound2less(mf2l));
    beta2best=beta2swipe(maxfound2more(mf2m));
    
    
    [minvar1,minvar2,maxvar1,maxvar2,alpha1best,alpha2best,beta1best,beta2best]=findextrema(var1,Var1low,var2,Var2low,alpha1best,alpha2best,beta1best,beta2best,0);
end

function [xround]=rounddec(x,prec)
xround=real(round((10^(prec-floor(log10(x)))*x))/(10^(prec-floor(log10(x)))));%real to avoid numerical errors that cause imaginary values...


% mindistA(i,1)=minvar1(find(abs(DistmatA(:,i))==min(abs(DistmatA(:,i)))),2);
% mindistA(i,2)=Distance1(mindistA(i,1));
% mindistA(i,3)=minvar2(i,2);
% mindistA(i,4)=Distance2(mindistA(i,3));
% mindistA(i,5)=DistmatA(find(abs(DistmatA(:,i))==min(abs(DistmatA(:,i)))),i);

function [mindistX]=DelDuplicates(mindistX,indexdup);
[uniqueX,ifi,jfi] = unique(mindistX(:,indexdup),'first');
[uniqueX,ila,jla] = unique(mindistX(:,indexdup),'last');
if indexdup==3, indexnotdup=1;end
if indexdup==1, indexnotdup=3;end
indexdisdup=indexdup+1;indexdisnotdup=indexnotdup+1;
isDuplicate=find(ifi~=ila);
tokeep=zeros(size(isDuplicate));
isNotDuplicate=(1:length(mindistX(:,5)))';
for k=1:length(isDuplicate)
    DuplicateIndex=(ifi(isDuplicate(k)):ila(isDuplicate(k)))';
    tokeep(k)=DuplicateIndex(find(abs(mindistX(DuplicateIndex,5))==min(abs(mindistX(DuplicateIndex,5)))));
    isNotDuplicate=setdiff(isNotDuplicate,DuplicateIndex);
end
ToKeep=union(isNotDuplicate,tokeep);
mindistX=mindistX(ToKeep,:);